﻿;/------------------------
;|
;|
;| Compile as dll and name it AddOn_LoadFromSource.dll (or AddOn_LoadFromSource.so)
;|
;| Add-On for Dialog Design0R
;| Im/Export dialog file from a PB/SB Source.
;| ©HeX0R 2018
;|
;| Adds possibility to load/save dialogs directly from source codes to the Dialog Design0R
;| But in fact it is just an example, it will most likely not handle all possible code scenarios.
;| It will create a FileName.tmp file which will contain the original content.
;|
;| As soon as you try to load a *.pb, *.pbi, *.sb or *.sbi file via Dialog Design0R, this AddOn will be called.
;|
;|
;|
;/------------------------

; ----------------------------------------------------------------------------
; "THE BEER-WARE LICENSE":
; <HeX0R@coderbu.de> wrote this file. as long as you retain this notice you
; can do whatever you want with this stuff. If we meet some day, and you think
; this stuff is worth it, you can buy me a beer in return
; (see address on http://hex0rs.coderbu.de/).
; Or just go out and drink a few on your own/with your friends ;)
;=============================================================================



CompilerIf #PB_Compiler_DLL = #False
	CompilerError "Please compile as shared library!"
CompilerEndIf
EnableExplicit

;{ constants
Enumeration
	#Flag_None
	#Flag_Escaped
EndEnumeration
;}

;{ structures
Structure _LOADED_FILE_
	FileName.s
	FileContent.s
	DialogStartPos.i
	DialogEndPos.i
	Flags.i
	LineFeed.s
	LinePrefix.s
	LineSuffix.s
EndStructure
;}


ProcedureDLL AttachProcess(Instance)               ;Init variables here

	Global ResultString.s
	Global LF._LOADED_FILE_
	Global LastError.s = "no error"

EndProcedure

ProcedureDLL DetachProcess(Instance)               ;Free variables here (if necessary)

	;void
EndProcedure

Procedure.s ParseContent(XMLRaw.s)                 ;Internal procedure, will parse the XML string from a source file
	Protected XML.s, Found, i, j, k, Count, Line.s
	Dim Lines.s(0)

	;Find used LineFeed Character (needed for saving)
	Count       = CountString(XMLRaw, #CRLF$)
	LF\LineFeed = ""
	If Count > 0
		LF\LineFeed = #CRLF$
	Else
		Count = CountString(XMLRaw, #CR$)
		If Count > 0
			LF\LineFeed = #CR$
		Else
			Count = CountString(XMLRaw, #LF$)
			If Count > 0
				LF\LineFeed = #LF$
			EndIf
		EndIf
	EndIf
	;Find used prefix and suffix (needed for saving)
	If Count > 2
		j = -1
		For i = 1 To Count
			k = j
			j = FindString(XMLRaw, LF\LineFeed, j + 2)
		Next i
		Line          = Mid(XMLRaw, k + Len(LF\LineFeed), j - k - Len(LF\LineFeed))
		LF\LinePrefix = Left(Line, FindString(Line, #DQUOTE$, 1))
		For i = Len(Line) To 1 Step - 1
			If Mid(Line, i, 1) = #DQUOTE$
				LF\LineSuffix = Mid(Line, i)
				Break
			EndIf
		Next i
	EndIf
	;now get dialog lines and create XML string
	If CreateRegularExpression(0, "[ \t]*<.*>")
		Found = ExtractRegularExpression(0, XMLRaw, Lines())
		XML   = ""
		For i = 0 To Found - 1
			If LF\Flags = #Flag_Escaped
				Lines(i) = UnescapeString(Lines(i))
			EndIf
			If i = 0
				If UnescapeString(Lines(i)) <> Lines(i)
					LF\Flags = #Flag_Escaped
					Lines(i) = UnescapeString(Lines(i))
				Else
					LF\Flags = #Flag_None
				EndIf
				XML + Lines(i)
			Else
				XML + LF\LineFeed + Lines(i)
			EndIf
		Next
		FreeRegularExpression(0)
	EndIf

	ProcedureReturn XML
EndProcedure

ProcedureDLL AddOn_Init()                          ;Init will be called when DD starts, we have to return the file extensions we can handle (separated by |)

	LastError    = "no error"
	ResultString = "pb|pbi|sb|sbi" ;This addon handles all PB and SB source codes
	ProcedureReturn @ResultString
EndProcedure

ProcedureDLL AddOn_GetFileType(FileName.s)         ;Let DD know which kind of xml-definition this file needs (called right before AddOn_LoadXML is called).
	
	Select LCase(GetExtensionPart(FileName))
		Case "pb", "pbi"
			ResultString = "purebasic"
		Case "sb", "sbi"
			ResultString = "spiderbasic"
		Default
			ResultString = ""
	EndSelect
	
	ProcedureReturn @ResultString
EndProcedure

ProcedureDLL AddOn_LoadXML(FileName.s)             ;User wants to load a file using this AddOn (==> from PB/SB file)
	Protected File.s, XML.s, BOM, i, j
	
	;First reset all of our variables
	ResultString      = ""
	LastError         = "no error"
	LF\DialogEndPos   = 0
	LF\DialogStartPos = 0
	LF\FileContent    = ""
	LF\FileName       = ""
	LF\Flags          = #Flag_None
	LF\LineFeed       = ""
	LF\LinePrefix     = ""
	LF\LineSuffix     = ""

	If ReadFile(0, FileName) = 0
		LastError = "unable to load '" + FileName + "'!"
	Else
		BOM  = ReadStringFormat(0)
		File = ReadString(0, BOM | #PB_File_IgnoreEOL)
		CloseFile(0)

		;Get Start/Endpos of included dialog (if any)
		i = FindString(File, Chr(60) + "dialogs>", 1, #PB_String_NoCase)
		If i
			j = FindString(File, "</dialogs>", i + 7, #PB_String_NoCase)
			If j
				j                 = j + 10
				LF\FileContent    = File
				LF\FileName       = FileName
				LF\DialogStartPos = i
				LF\DialogEndPos   = j
				ResultString      = ParseContent(Mid(File, i, j - i)) ;<-- here we are doing all of the parsing stuff
			EndIf
		Else
			i = FindString(File, Chr(60) + "window", 1, #PB_String_NoCase)
			If i
				j = FindString(File, "</window>", i + 7, #PB_String_NoCase)
				If j
					j                 = j + 9
					LF\FileContent    = File
					LF\FileName       = FileName
					LF\DialogStartPos = i
					LF\DialogEndPos   = j
					ResultString      = ParseContent(Mid(File, i, j - i)) ;<-- here we are doing all of the parsing stuff
				EndIf
			EndIf
		EndIf
		If i = 0
			LastError = "no dialog data found!"
		Else
			;check if ResultString is a valid XML
			If CatchXML(0, @ResultString, StringByteLength(ResultString), 0, #PB_Unicode)
				If XMLStatus(0) <> #PB_XML_Success
					ResultString = ""
					LastError    = "No valid XML-Code found!"
				EndIf
				FreeXML(0)
			EndIf
		EndIf
	EndIf

	ProcedureReturn @ResultString
EndProcedure

ProcedureDLL AddOn_SaveXML(FileName.s, XML.s)      ;User wants to save a file using this AddOn (==> into a PB/SB file)
	Protected Result, BOM, Found, i
	Dim Lines.s(0)

	LastError = "no error"
	If FileName = ""
		LastError = "No filename provided!"
	ElseIf LF\FileName = "" Or FileName <> LF\FileName
		LastError = "Can't save a file which hasn't previously been loaded!"
	Else
		;make sure we don't save it into a wrong file! SaveAs will not work!

		If CreateRegularExpression(0, "[ \t]*<.*>")
			If OpenFile(0, FileName) = 0
				LastError = "Can't open '" + FileName + "' with write access!"
			Else
				BOM = ReadStringFormat(0)
				FileSeek(0, LF\DialogStartPos - 1, #PB_Relative)
				TruncateFile(0)
				
				If LF\Flags <> #Flag_Escaped
					XML   = ReplaceString(XML, #DQUOTE$, "'")
				EndIf
				Found = ExtractRegularExpression(0, XML, Lines())
				For i = 0 To Found - 1
					If LF\Flags = #Flag_Escaped
						Lines(i) = EscapeString(Lines(i))
					EndIf
					If i = 0
						WriteString(0, Lines(i) + LF\LineSuffix + LF\LineFeed, BOM)
					ElseIf i = Found - 1
						WriteString(0, LF\LinePrefix + Lines(i), BOM)
					Else
						WriteString(0, LF\LinePrefix + Lines(i) + LF\LineSuffix + LF\LineFeed, BOM)
					EndIf
				Next i
				WriteString(0, Mid(LF\FileContent, LF\DialogEndPos), BOM)
				CloseFile(0)
				If CreateFile(1, FileName + ".tmp")
					WriteStringFormat(1, BOM)
					WriteString(1, LF\FileContent, BOM)
					CloseFile(1)
				EndIf
				Result = #True
			EndIf
			FreeRegularExpression(0)
		EndIf
	EndIf

	ProcedureReturn Result
EndProcedure

ProcedureDLL AddOn_GetLastError()                  ;Return last error
	
	ProcedureReturn @LastError
EndProcedure
; IDE Options = PureBasic 5.62 (Windows - x86)
; ExecutableFormat = Shared dll
; CursorPosition = 66
; Folding = Aw
; EnableXP
; EnableUser
; Executable = AddOn_LoadPB.dll
; CompileSourceDirectory
; EnableCompileCount = 42
; EnableBuildCount = 35
; EnableExeConstant